interface SessionObject {
    id: string;
    /**
     * Start time of current session
     */
    started: number;
    /**
     * Last known activity of the session
     */
    lastActivity: number;
    /**
     * Segment ID for replay events
     */
    segmentId: number;
    /**
     * Is the session sampled?
     */
    sampled: boolean;
}
interface SessionOptions {
    stickySession?: boolean;
    samplingRate?: number;
}
export declare class Session {
    /**
     * Session ID
     */
    private _id;
    /**
     * Start time of current session
     */
    private _started;
    /**
     * Last known activity of the session
     */
    private _lastActivity;
    /**
     * Sequence ID specific to replay updates
     */
    private _segmentId;
    /**
     * Previous session ID
     */
    private _previousSessionId;
    /**
     * Is the Session sampled?
     */
    private _sampled;
    readonly options: Required<Pick<SessionOptions, 'stickySession'>>;
    constructor(session?: Partial<SessionObject>, { stickySession, samplingRate }?: SessionOptions);
    get id(): string;
    get started(): number;
    get lastActivity(): number;
    set lastActivity(newDate: number);
    get segmentId(): number;
    set segmentId(id: number);
    get previousSessionId(): string;
    set previousSessionId(id: string);
    get sampled(): boolean;
    set sampled(_isSampled: boolean);
    toJSON(): SessionObject;
}
export {};
